'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var page = _interopDefault(require('page'));
var PropTypes = _interopDefault(require('prop-types'));

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var Router = function (_React$PureComponent) {
  inherits(Router, _React$PureComponent);

  function Router() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, Router);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Router.__proto__ || Object.getPrototypeOf(Router)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      path: null,
      ctx: null
    }, _this.onChange = function () {
      var onChange = _this.props.onChange;
      var _this$state = _this.state,
          path = _this$state.path,
          ctx = _this$state.ctx;


      if (onChange) onChange(path, ctx);
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(Router, [{
    key: 'componentWillMount',
    value: function componentWillMount() {
      var _this2 = this;

      var _props = this.props,
          routes = _props.routes,
          opts = _props.opts,
          base = _props.base;


      if (base) page.base(base);

      Object.keys(routes).forEach(function (path) {
        return page(path, function (ctx) {
          return _this2.setState({ path: path, ctx: ctx });
        });
      });

      page.start(opts);
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.onChange();
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      this.onChange();
    }
  }, {
    key: 'render',
    value: function render() {
      var routes = this.props.routes;
      var _state = this.state,
          path = _state.path,
          ctx = _state.ctx;


      var Route = routes[path];

      if (!Route && !routes['*'] || path === null) return null;

      return React.createElement(Route, { ctx: ctx });
    }
  }]);
  return Router;
}(React.PureComponent);

Router.propTypes = {
  routes: PropTypes.object.isRequired,
  opts: PropTypes.object,
  base: PropTypes.string,
  onChange: PropTypes.func
};
Router.defaultProps = {
  opts: {},
  base: '',
  onChange: undefined
};


var navigate = function navigate(to) {
  return page.show(to);
};
var redirect = page.redirect;
var Link = function Link(_ref2) {
  var children = _ref2.children,
      base = _ref2.base,
      href = _ref2.href,
      to = _ref2.to,
      props = objectWithoutProperties(_ref2, ['children', 'base', 'href', 'to']);
  return React.createElement(
    'a',
    _extends({
      href: '' + base + (href || to),
      onClick: function onClick(e) {
        navigate(href || to);
        e.preventDefault();
        return false;
      }
    }, props),
    children
  );
};
var hrefOrTo = function hrefOrTo(_ref3) {
  var href = _ref3.href,
      to = _ref3.to;
  return !href && !to && new Error('One of `href` or `to` props is required by the Link component.');
};

Link.propTypes = {
  base: PropTypes.string,
  children: PropTypes.any.isRequired,
  href: hrefOrTo,
  to: hrefOrTo
};

Link.defaultProps = {
  base: '',
  href: undefined,
  to: undefined
};

exports.default = Router;
exports.navigate = navigate;
exports.redirect = redirect;
exports.Link = Link;
